# eslint-plugin-sonarjs [![npm version](https://badge.fury.io/js/eslint-plugin-sonarjs.svg)](https://badge.fury.io/js/eslint-plugin-sonarjs)

SonarJS rules for ESLint to help developers produce [Clean Code](https://www.sonarsource.com/solutions/clean-code/) by detecting bugs and suspicious patterns.

## Rules

<!-- begin auto-generated rules list -->

💼 Configurations enabled in.\
🚫 Configurations disabled in.\
✅ Set in the `recommended` configuration.\
🔧 Automatically fixable by the [`--fix` CLI option](https://eslint.org/docs/user-guide/command-line-interface#--fix).\
💡 Manually fixable by [editor suggestions](https://eslint.org/docs/latest/use/core-concepts#rule-suggestions).\
💭 Requires [type information](https://typescript-eslint.io/linting/typed-linting).\
❌ Deprecated.

| Name                                                                                                    | Description                                                                                                                    | 💼  | 🚫  | 🔧  | 💡  | 💭  | ❌  |
| :------------------------------------------------------------------------------------------------------ | :----------------------------------------------------------------------------------------------------------------------------- | :-- | :-- | :-- | :-- | :-- | :-- |
| [accessor-pairs](https://sonarsource.github.io/rspec/#/rspec/S2376/javascript)                          | Property getters and setters should come in pairs                                                                              |     | ✅  |     |     |     |     |
| [alt-text](https://sonarsource.github.io/rspec/#/rspec/S1077/javascript)                                | Image, area, button with image and object elements should have an alternative text                                             | ✅  |     |     |     |     |     |
| [anchor-has-content](https://sonarsource.github.io/rspec/#/rspec/S6827/javascript)                      | Anchors should contain accessible content                                                                                      | ✅  |     |     | 💡  |     |     |
| [anchor-is-valid](https://sonarsource.github.io/rspec/#/rspec/S6844/javascript)                         | Anchor tags should not be used as buttons                                                                                      | ✅  |     |     |     |     |     |
| [anchor-precedence](https://sonarsource.github.io/rspec/#/rspec/S5850/javascript)                       | Alternatives in regular expressions should be grouped when used with anchors                                                   | ✅  |     |     |     | 💭  |     |
| [argument-type](https://sonarsource.github.io/rspec/#/rspec/S3782/javascript)                           | Arguments to built-in functions should match documented types                                                                  | ✅  |     |     |     | 💭  |     |
| [arguments-order](https://sonarsource.github.io/rspec/#/rspec/S2234/javascript)                         | Parameters should be passed in the correct order                                                                               | ✅  |     |     |     | 💭  |     |
| [arguments-usage](https://sonarsource.github.io/rspec/#/rspec/S3513/javascript)                         | "arguments" should not be accessed directly                                                                                    |     | ✅  |     |     |     |     |
| [array-callback-without-return](https://sonarsource.github.io/rspec/#/rspec/S3796/javascript)           | Callbacks of array methods should have return statements                                                                       | ✅  |     |     |     | 💭  |     |
| [array-constructor](https://sonarsource.github.io/rspec/#/rspec/S1528/javascript)                       | Array constructors should not be used                                                                                          |     | ✅  | 🔧  | 💡  |     |     |
| [arrow-function-convention](https://sonarsource.github.io/rspec/#/rspec/S3524/javascript)               | Braces and parentheses should be used consistently with arrow functions                                                        |     | ✅  |     |     |     |     |
| [assertions-in-tests](https://sonarsource.github.io/rspec/#/rspec/S2699/javascript)                     | Tests should include assertions                                                                                                | ✅  |     |     |     |     |     |
| [aws-apigateway-public-api](https://sonarsource.github.io/rspec/#/rspec/S6333/javascript)               | Creating public APIs is security-sensitive                                                                                     | ✅  |     |     |     |     |     |
| [aws-ec2-rds-dms-public](https://sonarsource.github.io/rspec/#/rspec/S6329/javascript)                  | Allowing public network access to cloud resources is security-sensitive                                                        | ✅  |     |     |     |     |     |
| [aws-ec2-unencrypted-ebs-volume](https://sonarsource.github.io/rspec/#/rspec/S6275/javascript)          | Using unencrypted EBS volumes is security-sensitive                                                                            | ✅  |     |     |     |     |     |
| [aws-efs-unencrypted](https://sonarsource.github.io/rspec/#/rspec/S6332/javascript)                     | Using unencrypted EFS file systems is security-sensitive                                                                       | ✅  |     |     |     |     |     |
| [aws-iam-all-privileges](https://sonarsource.github.io/rspec/#/rspec/S6302/javascript)                  | Policies granting all privileges are security-sensitive                                                                        | ✅  |     |     |     |     |     |
| [aws-iam-all-resources-accessible](https://sonarsource.github.io/rspec/#/rspec/S6304/javascript)        | Policies granting access to all resources of an account are security-sensitive                                                 |     | ✅  |     |     |     |     |
| [aws-iam-privilege-escalation](https://sonarsource.github.io/rspec/#/rspec/S6317/javascript)            | AWS IAM policies should limit the scope of permissions given                                                                   | ✅  |     |     |     |     |     |
| [aws-iam-public-access](https://sonarsource.github.io/rspec/#/rspec/S6270/javascript)                   | Policies authorizing public access to resources are security-sensitive                                                         | ✅  |     |     |     |     |     |
| [aws-opensearchservice-domain](https://sonarsource.github.io/rspec/#/rspec/S6308/javascript)            | Using unencrypted Elasticsearch domains is security-sensitive                                                                  | ✅  |     |     |     |     |     |
| [aws-rds-unencrypted-databases](https://sonarsource.github.io/rspec/#/rspec/S6303/javascript)           | Using unencrypted RDS DB resources is security-sensitive                                                                       | ✅  |     |     |     |     |     |
| [aws-restricted-ip-admin-access](https://sonarsource.github.io/rspec/#/rspec/S6321/javascript)          | Administration services access should be restricted to specific IP addresses                                                   | ✅  |     |     |     |     |     |
| [aws-s3-bucket-granted-access](https://sonarsource.github.io/rspec/#/rspec/S6265/javascript)            | Granting access to S3 buckets to all or authenticated users is security-sensitive                                              | ✅  |     |     |     |     |     |
| [aws-s3-bucket-insecure-http](https://sonarsource.github.io/rspec/#/rspec/S6249/javascript)             | Authorizing HTTP communications with S3 buckets is security-sensitive                                                          | ✅  |     |     |     |     |     |
| [aws-s3-bucket-public-access](https://sonarsource.github.io/rspec/#/rspec/S6281/javascript)             | Allowing public ACLs or policies on a S3 bucket is security-sensitive                                                          | ✅  |     |     |     |     |     |
| [aws-s3-bucket-server-encryption](https://sonarsource.github.io/rspec/#/rspec/S6245/javascript)         | Disabling server-side encryption of S3 buckets is security-sensitive                                                           |     | ✅  |     |     |     | ❌  |
| [aws-s3-bucket-versioning](https://sonarsource.github.io/rspec/#/rspec/S6252/javascript)                | Disabling versioning of S3 buckets is security-sensitive                                                                       | ✅  |     |     |     |     |     |
| [aws-sagemaker-unencrypted-notebook](https://sonarsource.github.io/rspec/#/rspec/S6319/javascript)      | Using unencrypted SageMaker notebook instances is security-sensitive                                                           | ✅  |     |     |     |     |     |
| [aws-sns-unencrypted-topics](https://sonarsource.github.io/rspec/#/rspec/S6327/javascript)              | Using unencrypted SNS topics is security-sensitive                                                                             | ✅  |     |     |     |     |     |
| [aws-sqs-unencrypted-queue](https://sonarsource.github.io/rspec/#/rspec/S6330/javascript)               | Using unencrypted SQS queues is security-sensitive                                                                             | ✅  |     |     |     |     |     |
| [bitwise-operators](https://sonarsource.github.io/rspec/#/rspec/S1529/javascript)                       | Bitwise operators should not be used in boolean contexts                                                                       | ✅  |     |     |     |     |     |
| [bool-param-default](https://sonarsource.github.io/rspec/#/rspec/S4798/javascript)                      | Optional boolean parameters should have default value                                                                          |     | ✅  |     |     |     |     |
| [brace-style](https://sonarsource.github.io/rspec/#/rspec/S1105/javascript)                             | An open curly brace should be located at the end of a line                                                                     |     | ✅  | 🔧  |     |     | ❌  |
| [call-argument-line](https://sonarsource.github.io/rspec/#/rspec/S1472/javascript)                      | Function call arguments should not start on new lines                                                                          | ✅  |     |     |     |     |     |
| [certificate-transparency](https://sonarsource.github.io/rspec/#/rspec/S5742/javascript)                | Disabling Certificate Transparency monitoring is security-sensitive                                                            | ✅  |     |     |     |     |     |
| [chai-determinate-assertion](https://sonarsource.github.io/rspec/#/rspec/S6092/javascript)              | Chai assertions should have only one reason to succeed                                                                         | ✅  |     |     |     |     |     |
| [class-name](https://sonarsource.github.io/rspec/#/rspec/S101/javascript)                               | Class names should comply with a naming convention                                                                             | ✅  |     |     |     |     |     |
| [class-prototype](https://sonarsource.github.io/rspec/#/rspec/S3525/javascript)                         | Class methods should be used instead of "prototype" assignments                                                                |     | ✅  |     |     | 💭  |     |
| [code-eval](https://sonarsource.github.io/rspec/#/rspec/S1523/javascript)                               | Dynamically executing code is security-sensitive                                                                               | ✅  |     |     |     |     |     |
| [cognitive-complexity](https://sonarsource.github.io/rspec/#/rspec/S3776/javascript)                    | Cognitive Complexity of functions should not be too high                                                                       | ✅  |     |     |     |     |     |
| [comma-or-logical-or-case](https://sonarsource.github.io/rspec/#/rspec/S3616/javascript)                | Comma and logical OR operators should not be used in switch cases                                                              | ✅  |     |     |     |     |     |
| [comment-regex](https://sonarsource.github.io/rspec/#/rspec/S124/javascript)                            | Track comments matching a regular expression                                                                                   |     | ✅  |     |     |     |     |
| [concise-regex](https://sonarsource.github.io/rspec/#/rspec/S6353/javascript)                           | Regular expression quantifiers and character classes should be used concisely                                                  | ✅  |     |     |     | 💭  |     |
| [conditional-indentation](https://sonarsource.github.io/rspec/#/rspec/S3973/javascript)                 | A conditionally executed single line should be denoted by indentation                                                          |     | ✅  |     |     |     | ❌  |
| [confidential-information-logging](https://sonarsource.github.io/rspec/#/rspec/S5757/javascript)        | Allowing confidential information to be logged is security-sensitive                                                           | ✅  |     |     |     |     |     |
| [constructor-for-side-effects](https://sonarsource.github.io/rspec/#/rspec/S1848/javascript)            | Objects should not be created to be dropped immediately without being used                                                     | ✅  |     |     |     |     |     |
| [content-length](https://sonarsource.github.io/rspec/#/rspec/S5693/javascript)                          | Allowing requests with excessive content length is security-sensitive                                                          | ✅  |     |     |     |     |     |
| [content-security-policy](https://sonarsource.github.io/rspec/#/rspec/S5728/javascript)                 | Disabling content security policy fetch directives is security-sensitive                                                       | ✅  |     |     |     |     |     |
| [cookie-no-httponly](https://sonarsource.github.io/rspec/#/rspec/S3330/javascript)                      | Creating cookies without the "HttpOnly" flag is security-sensitive                                                             | ✅  |     |     |     |     |     |
| [cookies](https://sonarsource.github.io/rspec/#/rspec/S2255/javascript)                                 | Writing cookies is security-sensitive                                                                                          |     | ✅  |     |     |     | ❌  |
| [cors](https://sonarsource.github.io/rspec/#/rspec/S5122/javascript)                                    | Having a permissive Cross-Origin Resource Sharing policy is security-sensitive                                                 | ✅  |     |     |     |     |     |
| [csrf](https://sonarsource.github.io/rspec/#/rspec/S4502/javascript)                                    | Disabling CSRF protections is security-sensitive                                                                               | ✅  |     |     |     |     |     |
| [cyclomatic-complexity](https://sonarsource.github.io/rspec/#/rspec/S1541/javascript)                   | Cyclomatic Complexity of functions should not be too high                                                                      |     | ✅  |     |     |     |     |
| [declarations-in-global-scope](https://sonarsource.github.io/rspec/#/rspec/S3798/javascript)            | Variables and functions should not be declared in the global scope                                                             |     | ✅  |     |     |     |     |
| [default-param-last](https://sonarsource.github.io/rspec/#/rspec/S1788/javascript)                      | Function parameters with default values should be last                                                                         | ✅  |     |     |     |     |     |
| [deprecation](https://sonarsource.github.io/rspec/#/rspec/S1874/javascript)                             | Deprecated APIs should not be used                                                                                             | ✅  |     |     |     | 💭  |     |
| [destructuring-assignment-syntax](https://sonarsource.github.io/rspec/#/rspec/S3514/javascript)         | Destructuring syntax should be used for assignments                                                                            |     | ✅  |     |     |     |     |
| [different-types-comparison](https://sonarsource.github.io/rspec/#/rspec/S3403/javascript)              | Strict equality operators should not be used with dissimilar types                                                             | ✅  |     | 🔧  | 💡  | 💭  |     |
| [disabled-auto-escaping](https://sonarsource.github.io/rspec/#/rspec/S5247/javascript)                  | Disabling auto-escaping in template engines is security-sensitive                                                              | ✅  |     |     |     | 💭  |     |
| [disabled-resource-integrity](https://sonarsource.github.io/rspec/#/rspec/S5725/javascript)             | Using remote artifacts without integrity checks is security-sensitive                                                          | ✅  |     |     |     | 💭  |     |
| [disabled-timeout](https://sonarsource.github.io/rspec/#/rspec/S6080/javascript)                        | Disabling Mocha timeouts should be explicit                                                                                    | ✅  |     |     |     |     |     |
| [dns-prefetching](https://sonarsource.github.io/rspec/#/rspec/S5743/javascript)                         | Allowing browsers to perform DNS prefetching is security-sensitive                                                             |     | ✅  |     |     |     | ❌  |
| [duplicates-in-character-class](https://sonarsource.github.io/rspec/#/rspec/S5869/javascript)           | Character classes in regular expressions should not contain the same character twice                                           | ✅  |     |     |     | 💭  |     |
| [elseif-without-else](https://sonarsource.github.io/rspec/#/rspec/S126/javascript)                      | "if ... else if" constructs should end with "else" clauses                                                                     |     | ✅  |     |     |     |     |
| [empty-string-repetition](https://sonarsource.github.io/rspec/#/rspec/S5842/javascript)                 | Repeated patterns in regular expressions should not match the empty string                                                     | ✅  |     |     |     | 💭  |     |
| [encryption](https://sonarsource.github.io/rspec/#/rspec/S4787/javascript)                              | Encrypting data is security-sensitive                                                                                          |     | ✅  |     |     |     | ❌  |
| [encryption-secure-mode](https://sonarsource.github.io/rspec/#/rspec/S5542/javascript)                  | Encryption algorithms should be used with secure mode and padding scheme                                                       | ✅  |     |     |     |     |     |
| [enforce-trailing-comma](https://sonarsource.github.io/rspec/#/rspec/S3723/javascript)                  | Trailing commas should be used                                                                                                 |     | ✅  | 🔧  |     |     | ❌  |
| [existing-groups](https://sonarsource.github.io/rspec/#/rspec/S6328/javascript)                         | Replacement strings should reference existing regular expression groups                                                        | ✅  |     |     |     | 💭  |     |
| [expression-complexity](https://sonarsource.github.io/rspec/#/rspec/S1067/javascript)                   | Expressions should not be too complex                                                                                          |     | ✅  |     |     |     |     |
| [file-header](https://sonarsource.github.io/rspec/#/rspec/S1451/javascript)                             | Track lack of copyright and license headers                                                                                    |     | ✅  |     |     |     |     |
| [file-name-differ-from-class](https://sonarsource.github.io/rspec/#/rspec/S3317/javascript)             | Default export names and file names should match                                                                               |     | ✅  |     |     |     |     |
| [file-permissions](https://sonarsource.github.io/rspec/#/rspec/S2612/javascript)                        | Setting loose POSIX file permissions is security-sensitive                                                                     | ✅  |     |     |     |     |     |
| [file-uploads](https://sonarsource.github.io/rspec/#/rspec/S2598/javascript)                            | File uploads should be restricted                                                                                              | ✅  |     |     |     |     |     |
| [fixme-tag](https://sonarsource.github.io/rspec/#/rspec/S1134/javascript)                               | Track uses of "FIXME" tags                                                                                                     | ✅  |     |     |     |     |     |
| [for-in](https://sonarsource.github.io/rspec/#/rspec/S1535/javascript)                                  | "for...in" loops should filter properties before acting on them                                                                |     | ✅  |     |     |     |     |
| [for-loop-increment-sign](https://sonarsource.github.io/rspec/#/rspec/S2251/javascript)                 | A "for" loop update clause should move the counter in the right direction                                                      | ✅  |     |     |     |     |     |
| [frame-ancestors](https://sonarsource.github.io/rspec/#/rspec/S5732/javascript)                         | Disabling content security policy frame-ancestors directive is security-sensitive                                              | ✅  |     |     |     |     |     |
| [function-inside-loop](https://sonarsource.github.io/rspec/#/rspec/S1515/javascript)                    | Functions should not be defined inside loops                                                                                   | ✅  |     |     |     |     |     |
| [function-name](https://sonarsource.github.io/rspec/#/rspec/S100/javascript)                            | Function and method names should comply with a naming convention                                                               |     | ✅  |     |     |     |     |
| [function-return-type](https://sonarsource.github.io/rspec/#/rspec/S3800/javascript)                    | Functions should always return the same type                                                                                   | ✅  |     |     |     | 💭  |     |
| [future-reserved-words](https://sonarsource.github.io/rspec/#/rspec/S1527/javascript)                   | Future reserved words should not be used as identifiers                                                                        | ✅  |     |     |     |     |     |
| [generator-without-yield](https://sonarsource.github.io/rspec/#/rspec/S3531/javascript)                 | Generators should explicitly "yield" a value                                                                                   | ✅  |     |     |     |     |     |
| [hashing](https://sonarsource.github.io/rspec/#/rspec/S4790/javascript)                                 | Using weak hashing algorithms is security-sensitive                                                                            | ✅  |     |     |     |     |     |
| [hidden-files](https://sonarsource.github.io/rspec/#/rspec/S5691/javascript)                            | Statically serving hidden files is security-sensitive                                                                          | ✅  |     |     |     |     |     |
| [hook-use-state](https://sonarsource.github.io/rspec/#/rspec/S6754/javascript)                          | The return value of "useState" should be destructured and named symmetrically                                                  | ✅  |     | 🔧  | 💡  |     |     |
| [html-has-lang](https://sonarsource.github.io/rspec/#/rspec/S5254/javascript)                           | HTML elements should have a valid language attribute                                                                           | ✅  |     |     | 💡  |     |     |
| [in-operator-type-error](https://sonarsource.github.io/rspec/#/rspec/S3785/javascript)                  | "in" should not be used with primitive types                                                                                   | ✅  |     |     |     | 💭  |     |
| [inconsistent-function-call](https://sonarsource.github.io/rspec/#/rspec/S3686/javascript)              | Functions should be called consistently with or without "new"                                                                  | ✅  |     |     |     |     |     |
| [index-of-compare-to-positive-number](https://sonarsource.github.io/rspec/#/rspec/S2692/javascript)     | "indexOf" checks should not be for positive numbers                                                                            | ✅  |     |     |     | 💭  |     |
| [insecure-cookie](https://sonarsource.github.io/rspec/#/rspec/S2092/javascript)                         | Creating cookies without the "secure" flag is security-sensitive                                                               | ✅  |     |     |     |     |     |
| [insecure-jwt-token](https://sonarsource.github.io/rspec/#/rspec/S5659/javascript)                      | JWT should be signed and verified with strong cipher algorithms                                                                | ✅  |     |     |     |     |     |
| [inverted-assertion-arguments](https://sonarsource.github.io/rspec/#/rspec/S3415/javascript)            | Assertion arguments should be passed in the correct order                                                                      | ✅  |     | 🔧  | 💡  |     |     |
| [jsx-key](https://sonarsource.github.io/rspec/#/rspec/S6477/javascript)                                 | JSX list components should have a key property                                                                                 | ✅  |     |     |     |     |     |
| [jsx-no-constructed-context-values](https://sonarsource.github.io/rspec/#/rspec/S6481/javascript)       | React Context Provider values should have stable identities                                                                    | ✅  |     |     |     |     |     |
| [jsx-no-useless-fragment](https://sonarsource.github.io/rspec/#/rspec/S6749/javascript)                 | Redundant React fragments should be removed                                                                                    | ✅  |     | 🔧  | 💡  |     |     |
| [label-has-associated-control](https://sonarsource.github.io/rspec/#/rspec/S6853/javascript)            | Label elements should have a text label and an associated control                                                              | ✅  |     |     |     |     |     |
| [label-position](https://sonarsource.github.io/rspec/#/rspec/S1439/javascript)                          | Only "while", "do", "for" and "switch" statements should be labelled                                                           | ✅  |     |     |     |     |     |
| [link-with-target-blank](https://sonarsource.github.io/rspec/#/rspec/S5148/javascript)                  | Authorizing an opened window to access back to the originating window is security-sensitive                                    | ✅  |     |     |     |     |     |
| [max-switch-cases](https://sonarsource.github.io/rspec/#/rspec/S1479/javascript)                        | "switch" statements should not have too many "case" clauses                                                                    | ✅  |     |     |     |     |     |
| [max-union-size](https://sonarsource.github.io/rspec/#/rspec/S4622/javascript)                          | Union types should not have too many elements                                                                                  |     | ✅  |     |     |     |     |
| [media-has-caption](https://sonarsource.github.io/rspec/#/rspec/S4084/javascript)                       | Media elements should have captions                                                                                            | ✅  |     |     |     |     |     |
| [misplaced-loop-counter](https://sonarsource.github.io/rspec/#/rspec/S1994/javascript)                  | "for" loop increment clauses should modify the loops' counters                                                                 | ✅  |     |     |     |     |     |
| [mouse-events-a11y](https://sonarsource.github.io/rspec/#/rspec/S1082/javascript)                       | Mouse events should have corresponding keyboard events                                                                         | ✅  |     |     | 💡  |     |     |
| [nested-control-flow](https://sonarsource.github.io/rspec/#/rspec/S134/javascript)                      | Control flow statements "if", "for", "while", "switch" and "try" should not be nested too deeply                               |     | ✅  |     |     |     |     |
| [new-cap](https://sonarsource.github.io/rspec/#/rspec/S2430/javascript)                                 | Constructor names should start with an upper case letter                                                                       | ✅  |     |     |     |     |     |
| [new-operator-misuse](https://sonarsource.github.io/rspec/#/rspec/S2999/javascript)                     | "new" should only be used with functions and classes                                                                           | ✅  |     |     |     | 💭  |     |
| [no-accessor-field-mismatch](https://sonarsource.github.io/rspec/#/rspec/S4275/javascript)              | Getters and setters should access the expected fields                                                                          | ✅  |     |     |     |     |     |
| [no-all-duplicated-branches](https://sonarsource.github.io/rspec/#/rspec/S3923/javascript)              | All branches in a conditional structure should not have exactly the same implementation                                        | ✅  |     |     |     |     |     |
| [no-alphabetical-sort](https://sonarsource.github.io/rspec/#/rspec/S2871/javascript)                    | "Array.prototype.sort()" and "Array.prototype.toSorted()" should use a compare function                                        | ✅  |     | 🔧  | 💡  | 💭  |     |
| [no-angular-bypass-sanitization](https://sonarsource.github.io/rspec/#/rspec/S6268/javascript)          | Disabling Angular built-in sanitization is security-sensitive                                                                  | ✅  |     |     |     |     |     |
| [no-array-delete](https://sonarsource.github.io/rspec/#/rspec/S2870/javascript)                         | "delete" should not be used on arrays                                                                                          | ✅  |     |     |     | 💭  |     |
| [no-array-index-key](https://sonarsource.github.io/rspec/#/rspec/S6479/javascript)                      | JSX list components should not use array indexes as key                                                                        | ✅  |     |     |     |     |     |
| [no-associative-arrays](https://sonarsource.github.io/rspec/#/rspec/S3579/javascript)                   | Array indexes should be numeric                                                                                                | ✅  |     |     |     | 💭  |     |
| [no-base-to-string](https://sonarsource.github.io/rspec/#/rspec/S6551/javascript)                       | Objects and classes converted or coerced to strings should define a "toString()" method                                        | ✅  |     |     |     | 💭  |     |
| [no-built-in-override](https://sonarsource.github.io/rspec/#/rspec/S2424/javascript)                    | Built-in objects should not be overridden                                                                                      |     | ✅  |     |     |     |     |
| [no-case-label-in-switch](https://sonarsource.github.io/rspec/#/rspec/S1219/javascript)                 | "switch" statements should not contain non-case labels                                                                         | ✅  |     |     |     |     |     |
| [no-clear-text-protocols](https://sonarsource.github.io/rspec/#/rspec/S5332/javascript)                 | Using clear-text protocols is security-sensitive                                                                               | ✅  |     |     |     |     |     |
| [no-code-after-done](https://sonarsource.github.io/rspec/#/rspec/S6079/javascript)                      | Tests should not execute any code after "done()" is called                                                                     | ✅  |     |     |     |     |     |
| [no-collapsible-if](https://sonarsource.github.io/rspec/#/rspec/S1066/javascript)                       | Mergeable "if" statements should be combined                                                                                   |     | ✅  |     |     |     |     |
| [no-collection-size-mischeck](https://sonarsource.github.io/rspec/#/rspec/S3981/javascript)             | Collection size and array length comparisons should make sense                                                                 | ✅  |     | 🔧  | 💡  | 💭  |     |
| [no-commented-code](https://sonarsource.github.io/rspec/#/rspec/S125/javascript)                        | Sections of code should not be commented out                                                                                   | ✅  |     | 🔧  | 💡  |     |     |
| [no-dead-store](https://sonarsource.github.io/rspec/#/rspec/S1854/javascript)                           | Unused assignments should be removed                                                                                           | ✅  |     |     |     |     |     |
| [no-delete-var](https://sonarsource.github.io/rspec/#/rspec/S3001/javascript)                           | "delete" should be used only with object properties                                                                            | ✅  |     |     |     |     |     |
| [no-deprecated-react](https://sonarsource.github.io/rspec/#/rspec/S6957/javascript)                     | Deprecated React APIs should not be used                                                                                       | ✅  |     |     |     |     |     |
| [no-duplicate-in-composite](https://sonarsource.github.io/rspec/#/rspec/S4621/javascript)               | Union and intersection types should not include duplicated constituents                                                        | ✅  |     | 🔧  | 💡  |     |     |
| [no-duplicate-string](https://sonarsource.github.io/rspec/#/rspec/S1192/javascript)                     | String literals should not be duplicated                                                                                       |     | ✅  |     |     |     |     |
| [no-duplicated-branches](https://sonarsource.github.io/rspec/#/rspec/S1871/javascript)                  | Two branches in a conditional structure should not have exactly the same implementation                                        | ✅  |     |     |     |     |     |
| [no-element-overwrite](https://sonarsource.github.io/rspec/#/rspec/S4143/javascript)                    | Collection elements should not be replaced unconditionally                                                                     | ✅  |     |     |     |     |     |
| [no-empty-after-reluctant](https://sonarsource.github.io/rspec/#/rspec/S6019/javascript)                | Reluctant quantifiers in regular expressions should be followed by an expression that can't match the empty string             | ✅  |     |     |     | 💭  |     |
| [no-empty-alternatives](https://sonarsource.github.io/rspec/#/rspec/S6323/javascript)                   | Alternation in regular expressions should not contain empty alternatives                                                       | ✅  |     |     |     | 💭  |     |
| [no-empty-collection](https://sonarsource.github.io/rspec/#/rspec/S4158/javascript)                     | Empty collections should not be accessed or iterated                                                                           | ✅  |     |     |     |     |     |
| [no-empty-function](https://sonarsource.github.io/rspec/#/rspec/S1186/javascript)                       | Functions should not be empty                                                                                                  | ✅  |     | 🔧  | 💡  |     |     |
| [no-empty-group](https://sonarsource.github.io/rspec/#/rspec/S6331/javascript)                          | Regular expressions should not contain empty groups                                                                            | ✅  |     |     |     | 💭  |     |
| [no-empty-interface](https://sonarsource.github.io/rspec/#/rspec/S4023/javascript)                      | Interfaces should not be empty                                                                                                 |     | ✅  | 🔧  | 💡  |     |     |
| [no-empty-test-file](https://sonarsource.github.io/rspec/#/rspec/S2187/javascript)                      | Test files should contain at least one test case                                                                               | ✅  |     |     |     |     |     |
| [no-equals-in-for-termination](https://sonarsource.github.io/rspec/#/rspec/S888/javascript)             | Equality operators should not be used in "for" loop termination conditions                                                     | ✅  |     |     |     |     |     |
| [no-exclusive-tests](https://sonarsource.github.io/rspec/#/rspec/S6426/javascript)                      | Exclusive tests should not be commited to version control                                                                      | ✅  |     | 🔧  | 💡  |     |     |
| [no-extend-native](https://sonarsource.github.io/rspec/#/rspec/S6643/javascript)                        | Prototypes of builtin objects should not be modified                                                                           | ✅  |     |     |     |     |     |
| [no-extra-arguments](https://sonarsource.github.io/rspec/#/rspec/S930/javascript)                       | Function calls should not pass extra arguments                                                                                 | ✅  |     |     |     |     |     |
| [no-extra-semi](https://sonarsource.github.io/rspec/#/rspec/S1116/javascript)                           | Extra semicolons should be removed                                                                                             |     | ✅  | 🔧  |     |     | ❌  |
| [no-find-dom-node](https://sonarsource.github.io/rspec/#/rspec/S6788/javascript)                        | React's "findDOMNode" should not be used                                                                                       | ✅  |     |     |     |     |     |
| [no-for-in-iterable](https://sonarsource.github.io/rspec/#/rspec/S4139/javascript)                      | "for in" should not be used with iterables                                                                                     |     | ✅  |     |     | 💭  |     |
| [no-function-declaration-in-block](https://sonarsource.github.io/rspec/#/rspec/S1530/javascript)        | Function declarations should not be made within blocks                                                                         |     | ✅  |     |     |     |     |
| [no-global-this](https://sonarsource.github.io/rspec/#/rspec/S2990/javascript)                          | The global "this" object should not be used                                                                                    | ✅  |     | 🔧  | 💡  |     |     |
| [no-globals-shadowing](https://sonarsource.github.io/rspec/#/rspec/S2137/javascript)                    | Special identifiers should not be bound or assigned                                                                            | ✅  |     |     |     |     |     |
| [no-gratuitous-expressions](https://sonarsource.github.io/rspec/#/rspec/S2589/javascript)               | Boolean expressions should not be gratuitous                                                                                   | ✅  |     |     |     |     |     |
| [no-hardcoded-credentials](https://sonarsource.github.io/rspec/#/rspec/S2068/javascript)                | Hard-coded credentials are security-sensitive                                                                                  | ✅  |     |     |     |     |     |
| [no-hardcoded-ip](https://sonarsource.github.io/rspec/#/rspec/S1313/javascript)                         | Using hardcoded IP addresses is security-sensitive                                                                             | ✅  |     |     |     |     |     |
| [no-hook-setter-in-body](https://sonarsource.github.io/rspec/#/rspec/S6442/javascript)                  | React's useState hook should not be used directly in the render function or body of a component                                | ✅  |     |     |     |     |     |
| [no-identical-conditions](https://sonarsource.github.io/rspec/#/rspec/S1862/javascript)                 | "if/else if" chains and "switch" cases should not have the same condition                                                      | ✅  |     |     |     |     |     |
| [no-identical-expressions](https://sonarsource.github.io/rspec/#/rspec/S1764/javascript)                | Identical expressions should not be used on both sides of a binary operator                                                    | ✅  |     |     |     |     |     |
| [no-identical-functions](https://sonarsource.github.io/rspec/#/rspec/S4144/javascript)                  | Functions should not have identical implementations                                                                            | ✅  |     |     |     |     |     |
| [no-ignored-exceptions](https://sonarsource.github.io/rspec/#/rspec/S2486/javascript)                   | Exceptions should not be ignored                                                                                               | ✅  |     |     |     |     |     |
| [no-ignored-return](https://sonarsource.github.io/rspec/#/rspec/S2201/javascript)                       | Return values from functions without side effects should not be ignored                                                        | ✅  |     |     |     | 💭  |     |
| [no-implicit-dependencies](https://sonarsource.github.io/rspec/#/rspec/S4328/javascript)                | Dependencies should be explicit                                                                                                |     | ✅  |     |     |     |     |
| [no-implicit-global](https://sonarsource.github.io/rspec/#/rspec/S2703/javascript)                      | Variables should be declared explicitly                                                                                        | ✅  |     |     |     |     |     |
| [no-in-misuse](https://sonarsource.github.io/rspec/#/rspec/S4619/javascript)                            | "in" should not be used on arrays                                                                                              | ✅  |     | 🔧  | 💡  | 💭  |     |
| [no-incomplete-assertions](https://sonarsource.github.io/rspec/#/rspec/S2970/javascript)                | Assertions should be complete                                                                                                  | ✅  |     |     |     |     |     |
| [no-inconsistent-returns](https://sonarsource.github.io/rspec/#/rspec/S3801/javascript)                 | Functions should use "return" consistently                                                                                     |     | ✅  |     |     |     |     |
| [no-incorrect-string-concat](https://sonarsource.github.io/rspec/#/rspec/S3402/javascript)              | Strings and non-strings should not be added                                                                                    |     | ✅  |     |     | 💭  |     |
| [no-infinite-loop](https://sonarsource.github.io/rspec/#/rspec/S2189/javascript)                        | Loops should not be infinite                                                                                                   | ✅  |     |     |     |     |     |
| [no-intrusive-permissions](https://sonarsource.github.io/rspec/#/rspec/S5604/javascript)                | Using intrusive permissions is security-sensitive                                                                              | ✅  |     |     |     |     |     |
| [no-invalid-await](https://sonarsource.github.io/rspec/#/rspec/S4123/javascript)                        | "await" should only be used with promises                                                                                      | ✅  |     |     |     | 💭  |     |
| [no-invariant-returns](https://sonarsource.github.io/rspec/#/rspec/S3516/javascript)                    | Function returns should not be invariant                                                                                       | ✅  |     |     |     |     |     |
| [no-inverted-boolean-check](https://sonarsource.github.io/rspec/#/rspec/S1940/javascript)               | Boolean checks should not be inverted                                                                                          | ✅  |     | 🔧  | 💡  |     |     |
| [no-ip-forward](https://sonarsource.github.io/rspec/#/rspec/S5759/javascript)                           | Forwarding client IP address is security-sensitive                                                                             | ✅  |     |     |     |     |     |
| [no-labels](https://sonarsource.github.io/rspec/#/rspec/S1119/javascript)                               | Labels should not be used                                                                                                      | ✅  |     |     |     |     |     |
| [no-literal-call](https://sonarsource.github.io/rspec/#/rspec/S6958/javascript)                         | Literals should not be used as functions                                                                                       | ✅  |     |     |     |     |     |
| [no-lonely-if](https://sonarsource.github.io/rspec/#/rspec/S6660/javascript)                            | If statements should not be the only statement in else blocks                                                                  | ✅  |     | 🔧  |     |     |     |
| [no-mime-sniff](https://sonarsource.github.io/rspec/#/rspec/S5734/javascript)                           | Allowing browsers to sniff MIME types is security-sensitive                                                                    | ✅  |     |     |     |     |     |
| [no-misleading-array-reverse](https://sonarsource.github.io/rspec/#/rspec/S4043/javascript)             | Array-mutating methods should not be used misleadingly                                                                         | ✅  |     | 🔧  | 💡  | 💭  |     |
| [no-misused-promises](https://sonarsource.github.io/rspec/#/rspec/S6544/javascript)                     | Promises should not be misused                                                                                                 | ✅  |     |     | 💡  | 💭  |     |
| [no-mixed-content](https://sonarsource.github.io/rspec/#/rspec/S5730/javascript)                        | Allowing mixed-content is security-sensitive                                                                                   | ✅  |     |     |     |     |     |
| [no-nested-assignment](https://sonarsource.github.io/rspec/#/rspec/S1121/javascript)                    | Assignments should not be made from within sub-expressions                                                                     | ✅  |     |     |     |     |     |
| [no-nested-conditional](https://sonarsource.github.io/rspec/#/rspec/S3358/javascript)                   | Ternary operators should not be nested                                                                                         | ✅  |     |     |     |     |     |
| [no-nested-functions](https://sonarsource.github.io/rspec/#/rspec/S2004/javascript)                     | Functions should not be nested too deeply                                                                                      | ✅  |     |     |     |     |     |
| [no-nested-incdec](https://sonarsource.github.io/rspec/#/rspec/S881/javascript)                         | Increment (++) and decrement (--) operators should not be used in a method call or mixed with other operators in an expression |     | ✅  |     |     |     |     |
| [no-nested-switch](https://sonarsource.github.io/rspec/#/rspec/S1821/javascript)                        | "switch" statements should not be nested                                                                                       |     | ✅  |     |     |     |     |
| [no-nested-template-literals](https://sonarsource.github.io/rspec/#/rspec/S4624/javascript)             | Template literals should not be nested                                                                                         | ✅  |     |     |     |     |     |
| [no-one-iteration-loop](https://sonarsource.github.io/rspec/#/rspec/S1751/javascript)                   | Loops with at most one iteration should be refactored                                                                          | ✅  |     |     |     |     |     |
| [no-os-command-from-path](https://sonarsource.github.io/rspec/#/rspec/S4036/javascript)                 | Searching OS commands in PATH is security-sensitive                                                                            | ✅  |     |     |     |     |     |
| [no-parameter-reassignment](https://sonarsource.github.io/rspec/#/rspec/S1226/javascript)               | Initial values of parameters, caught exceptions, and loop variables should not be ignored                                      | ✅  |     |     |     |     |     |
| [no-primitive-wrappers](https://sonarsource.github.io/rspec/#/rspec/S1533/javascript)                   | Wrapper objects should not be used for primitive types                                                                         | ✅  |     | 🔧  | 💡  |     |     |
| [no-redeclare](https://sonarsource.github.io/rspec/#/rspec/S2814/javascript)                            | Variables and functions should not be redeclared                                                                               | ✅  |     |     |     |     |     |
| [no-redundant-assignments](https://sonarsource.github.io/rspec/#/rspec/S4165/javascript)                | Assignments should not be redundant                                                                                            | ✅  |     |     |     |     |     |
| [no-redundant-boolean](https://sonarsource.github.io/rspec/#/rspec/S1125/javascript)                    | Boolean literals should not be used in comparisons                                                                             | ✅  |     |     |     |     |     |
| [no-redundant-jump](https://sonarsource.github.io/rspec/#/rspec/S3626/javascript)                       | Jump statements should not be redundant                                                                                        | ✅  |     | 🔧  | 💡  |     |     |
| [no-redundant-optional](https://sonarsource.github.io/rspec/#/rspec/S4782/javascript)                   | Optional property declarations should not use both '?' and 'undefined' syntax                                                  | ✅  |     | 🔧  | 💡  | 💭  |     |
| [no-redundant-parentheses](https://sonarsource.github.io/rspec/#/rspec/S1110/javascript)                | Redundant pairs of parentheses should be removed                                                                               |     | ✅  | 🔧  | 💡  |     | ❌  |
| [no-redundant-type-constituents](https://sonarsource.github.io/rspec/#/rspec/S6571/javascript)          | Type constituents of unions and intersections should not be redundant                                                          | ✅  |     |     |     | 💭  |     |
| [no-reference-error](https://sonarsource.github.io/rspec/#/rspec/S3827/javascript)                      | Variables should be defined before being used                                                                                  |     | ✅  |     |     |     |     |
| [no-referrer-policy](https://sonarsource.github.io/rspec/#/rspec/S5736/javascript)                      | Disabling strict HTTP no-referrer policy is security-sensitive                                                                 | ✅  |     |     |     |     |     |
| [no-require-or-define](https://sonarsource.github.io/rspec/#/rspec/S3533/javascript)                    | "import" should be used to include external code                                                                               |     | ✅  |     |     | 💭  |     |
| [no-return-type-any](https://sonarsource.github.io/rspec/#/rspec/S4324/javascript)                      | Primitive return types should be used                                                                                          |     | ✅  |     |     | 💭  |     |
| [no-same-argument-assert](https://sonarsource.github.io/rspec/#/rspec/S5863/javascript)                 | Assertions should not be given twice the same argument                                                                         | ✅  |     |     |     |     |     |
| [no-same-line-conditional](https://sonarsource.github.io/rspec/#/rspec/S3972/javascript)                | Conditionals should start on new lines                                                                                         | ✅  |     | 🔧  | 💡  |     |     |
| [no-self-compare](https://sonarsource.github.io/rspec/#/rspec/S6679/javascript)                         | "Number.isNaN()" should be used to check for "NaN" value                                                                       | ✅  |     | 🔧  | 💡  |     |     |
| [no-small-switch](https://sonarsource.github.io/rspec/#/rspec/S1301/javascript)                         | "switch" statements should have at least 3 "case" clauses                                                                      | ✅  |     |     |     |     |     |
| [no-tab](https://sonarsource.github.io/rspec/#/rspec/S105/javascript)                                   | Tabulation characters should not be used                                                                                       |     | ✅  |     |     |     | ❌  |
| [no-table-as-layout](https://sonarsource.github.io/rspec/#/rspec/S5257/javascript)                      | HTML "&lt;table&gt;" should not be used for layout purposes                                                                    | ✅  |     |     |     |     |     |
| [no-this-alias](https://sonarsource.github.io/rspec/#/rspec/S4327/javascript)                           | "this" should not be assigned to variables                                                                                     |     | ✅  |     |     |     |     |
| [no-throw-literal](https://sonarsource.github.io/rspec/#/rspec/S3696/javascript)                        | Literals should not be thrown                                                                                                  | ✅  |     | 🔧  | 💡  | 💭  |     |
| [no-try-promise](https://sonarsource.github.io/rspec/#/rspec/S4822/javascript)                          | Promise rejections should not be caught by "try" blocks                                                                        | ✅  |     |     |     | 💭  |     |
| [no-undefined-argument](https://sonarsource.github.io/rspec/#/rspec/S4623/javascript)                   | "undefined" should not be passed as the value of optional parameters                                                           | ✅  |     | 🔧  | 💡  | 💭  |     |
| [no-undefined-assignment](https://sonarsource.github.io/rspec/#/rspec/S2138/javascript)                 | "undefined" should not be assigned                                                                                             |     | ✅  |     |     |     |     |
| [no-unenclosed-multiline-block](https://sonarsource.github.io/rspec/#/rspec/S2681/javascript)           | Multiline blocks should be enclosed in curly braces                                                                            | ✅  |     |     |     |     |     |
| [no-uniq-key](https://sonarsource.github.io/rspec/#/rspec/S6486/javascript)                             | JSX list components keys should match up between renders                                                                       | ✅  |     |     |     |     |     |
| [no-unknown-property](https://sonarsource.github.io/rspec/#/rspec/S6747/javascript)                     | JSX elements should not use unknown properties and attributes                                                                  | ✅  |     | 🔧  | 💡  |     |     |
| [no-unreachable](https://sonarsource.github.io/rspec/#/rspec/S1763/javascript)                          | All code should be reachable                                                                                                   | ✅  |     | 🔧  | 💡  |     |     |
| [no-unsafe](https://sonarsource.github.io/rspec/#/rspec/S6791/javascript)                               | React legacy lifecycle methods should not be used                                                                              | ✅  |     |     |     |     |     |
| [no-unsafe-unzip](https://sonarsource.github.io/rspec/#/rspec/S5042/javascript)                         | Expanding archive files without controlling resource consumption is security-sensitive                                         | ✅  |     |     |     |     |     |
| [no-unstable-nested-components](https://sonarsource.github.io/rspec/#/rspec/S6478/javascript)           | React components should not be nested                                                                                          | ✅  |     |     |     |     |     |
| [no-unthrown-error](https://sonarsource.github.io/rspec/#/rspec/S3984/javascript)                       | Errors should not be created without being thrown                                                                              | ✅  |     | 🔧  | 💡  |     |     |
| [no-unused-collection](https://sonarsource.github.io/rspec/#/rspec/S4030/javascript)                    | Collection contents should be used                                                                                             | ✅  |     |     |     |     |     |
| [no-unused-expressions](https://sonarsource.github.io/rspec/#/rspec/S905/javascript)                    | Non-empty statements should change control flow or have at least one side-effect                                               | ✅  |     |     |     |     |     |
| [no-unused-function-argument](https://sonarsource.github.io/rspec/#/rspec/S1172/javascript)             | Unused function parameters should be removed                                                                                   |     | ✅  | 🔧  | 💡  |     |     |
| [no-unused-private-class-members](https://sonarsource.github.io/rspec/#/rspec/S1068/javascript)         | Unused private class members should be removed                                                                                 | ✅  |     | 🔧  | 💡  |     |     |
| [no-use-of-empty-return-value](https://sonarsource.github.io/rspec/#/rspec/S3699/javascript)            | The return value of void functions should not be used                                                                          | ✅  |     |     |     |     |     |
| [no-useless-call](https://sonarsource.github.io/rspec/#/rspec/S6676/javascript)                         | Calls to ".call()" and ".apply()" methods should not be redundant                                                              | ✅  |     | 🔧  | 💡  |     |     |
| [no-useless-catch](https://sonarsource.github.io/rspec/#/rspec/S2737/javascript)                        | "catch" clauses should do more than rethrow                                                                                    | ✅  |     |     |     |     |     |
| [no-useless-constructor](https://sonarsource.github.io/rspec/#/rspec/S6647/javascript)                  | Unnecessary constructors should be removed                                                                                     | ✅  |     | 🔧  | 💡  |     |     |
| [no-useless-increment](https://sonarsource.github.io/rspec/#/rspec/S2123/javascript)                    | Values should not be uselessly incremented                                                                                     | ✅  |     |     |     |     |     |
| [no-useless-intersection](https://sonarsource.github.io/rspec/#/rspec/S4335/javascript)                 | Type intersections should use meaningful types                                                                                 | ✅  |     |     |     | 💭  |     |
| [no-useless-react-setstate](https://sonarsource.github.io/rspec/#/rspec/S6443/javascript)               | React state setter function should not be called with its matching state variable                                              | ✅  |     |     |     |     |     |
| [no-var](https://sonarsource.github.io/rspec/#/rspec/S3504/javascript)                                  | Variables should be declared with "let" or "const"                                                                             | ✅  |     | 🔧  |     |     |     |
| [no-variable-usage-before-declaration](https://sonarsource.github.io/rspec/#/rspec/S1526/javascript)    | Variables declared with "var" should be declared before they are used                                                          |     | ✅  |     |     |     |     |
| [no-vue-bypass-sanitization](https://sonarsource.github.io/rspec/#/rspec/S6299/javascript)              | Disabling Vue.js built-in escaping is security-sensitive                                                                       | ✅  |     |     |     |     |     |
| [no-weak-cipher](https://sonarsource.github.io/rspec/#/rspec/S5547/javascript)                          | Cipher algorithms should be robust                                                                                             | ✅  |     |     |     |     |     |
| [no-weak-keys](https://sonarsource.github.io/rspec/#/rspec/S4426/javascript)                            | Cryptographic keys should be robust                                                                                            | ✅  |     |     |     |     |     |
| [no-wildcard-import](https://sonarsource.github.io/rspec/#/rspec/S2208/javascript)                      | Wildcard imports should not be used                                                                                            |     | ✅  |     |     |     |     |
| [non-existent-operator](https://sonarsource.github.io/rspec/#/rspec/S2757/javascript)                   | Non-existent operators '=+', '=-' and '=!' should not be used                                                                  | ✅  |     | 🔧  | 💡  |     |     |
| [non-number-in-arithmetic-expression](https://sonarsource.github.io/rspec/#/rspec/S3760/javascript)     | Arithmetic operators should only have numbers as operands                                                                      |     | ✅  |     |     | 💭  |     |
| [null-dereference](https://sonarsource.github.io/rspec/#/rspec/S2259/javascript)                        | Properties of variables with "null" or "undefined" values should not be accessed                                               | ✅  |     |     |     | 💭  |     |
| [object-alt-content](https://sonarsource.github.io/rspec/#/rspec/S5264/javascript)                      | "<object>" tags should provide an alternative content                                                                          | ✅  |     |     |     |     |     |
| [object-shorthand](https://sonarsource.github.io/rspec/#/rspec/S3498/javascript)                        | Object literal shorthand syntax should be used                                                                                 |     | ✅  | 🔧  |     |     |     |
| [operation-returning-nan](https://sonarsource.github.io/rspec/#/rspec/S3757/javascript)                 | Arithmetic operations should not result in "NaN"                                                                               |     | ✅  |     |     | 💭  |     |
| [os-command](https://sonarsource.github.io/rspec/#/rspec/S4721/javascript)                              | Using shell interpreter when executing OS commands is security-sensitive                                                       | ✅  |     |     |     |     |     |
| [post-message](https://sonarsource.github.io/rspec/#/rspec/S2819/javascript)                            | Origins should be verified during cross-origin communications                                                                  | ✅  |     |     |     | 💭  |     |
| [prefer-default-last](https://sonarsource.github.io/rspec/#/rspec/S4524/javascript)                     | "default" clauses should be last                                                                                               | ✅  |     |     |     |     |     |
| [prefer-enum-initializers](https://sonarsource.github.io/rspec/#/rspec/S6572/javascript)                | Enum member values should be either all initialized or none                                                                    | ✅  |     | 🔧  | 💡  |     |     |
| [prefer-for-of](https://sonarsource.github.io/rspec/#/rspec/S4138/javascript)                           | "for of" should be used with Iterables                                                                                         | ✅  |     | 🔧  | 💡  |     |     |
| [prefer-function-type](https://sonarsource.github.io/rspec/#/rspec/S6598/javascript)                    | Function types should be preferred                                                                                             | ✅  |     | 🔧  |     |     |     |
| [prefer-immediate-return](https://sonarsource.github.io/rspec/#/rspec/S1488/javascript)                 | Local variables should not be declared and then immediately returned or thrown                                                 |     | ✅  | 🔧  |     |     |     |
| [prefer-namespace-keyword](https://sonarsource.github.io/rspec/#/rspec/S4156/javascript)                | "module" should not be used                                                                                                    | ✅  |     | 🔧  |     |     |     |
| [prefer-nullish-coalescing](https://sonarsource.github.io/rspec/#/rspec/S6606/javascript)               | Nullish coalescing should be preferred                                                                                         | ✅  |     | 🔧  | 💡  | 💭  |     |
| [prefer-object-literal](https://sonarsource.github.io/rspec/#/rspec/S2428/javascript)                   | Object literal syntax should be used                                                                                           |     | ✅  |     |     |     |     |
| [prefer-object-spread](https://sonarsource.github.io/rspec/#/rspec/S6661/javascript)                    | Object spread syntax should be used instead of "Object.assign"                                                                 | ✅  |     | 🔧  |     |     |     |
| [prefer-promise-shorthand](https://sonarsource.github.io/rspec/#/rspec/S4634/javascript)                | Shorthand promises should be used                                                                                              | ✅  |     | 🔧  | 💡  |     |     |
| [prefer-single-boolean-return](https://sonarsource.github.io/rspec/#/rspec/S1126/javascript)            | Return of boolean expressions should not be wrapped into an "if-then-else" statement                                           | ✅  |     | 🔧  | 💡  |     |     |
| [prefer-spread](https://sonarsource.github.io/rspec/#/rspec/S6666/javascript)                           | Spread syntax should be used instead of "apply()"                                                                              | ✅  |     | 🔧  | 💡  |     |     |
| [prefer-string-starts-ends-with](https://sonarsource.github.io/rspec/#/rspec/S6557/javascript)          | Ends of strings should be checked with "startsWith()" and "endsWith()"                                                         | ✅  |     | 🔧  | 💡  | 💭  |     |
| [prefer-template](https://sonarsource.github.io/rspec/#/rspec/S3512/javascript)                         | Template strings should be used instead of concatenation                                                                       |     | ✅  | 🔧  |     |     |     |
| [prefer-type-guard](https://sonarsource.github.io/rspec/#/rspec/S4322/javascript)                       | Type predicates should be used                                                                                                 | ✅  |     | 🔧  | 💡  |     |     |
| [prefer-while](https://sonarsource.github.io/rspec/#/rspec/S1264/javascript)                            | A "while" loop should be used instead of a "for" loop                                                                          | ✅  |     | 🔧  |     |     |     |
| [process-argv](https://sonarsource.github.io/rspec/#/rspec/S4823/javascript)                            | Using command line arguments is security-sensitive                                                                             |     | ✅  |     |     |     | ❌  |
| [production-debug](https://sonarsource.github.io/rspec/#/rspec/S4507/javascript)                        | Delivering code in production with debug features activated is security-sensitive                                              | ✅  |     |     |     |     |     |
| [pseudo-random](https://sonarsource.github.io/rspec/#/rspec/S2245/javascript)                           | Using pseudorandom number generators (PRNGs) is security-sensitive                                                             | ✅  |     |     |     |     |     |
| [public-static-readonly](https://sonarsource.github.io/rspec/#/rspec/S1444/javascript)                  | Public "static" fields should be read-only                                                                                     | ✅  |     | 🔧  | 💡  |     |     |
| [publicly-writable-directories](https://sonarsource.github.io/rspec/#/rspec/S5443/javascript)           | Using publicly writable directories is security-sensitive                                                                      | ✅  |     |     |     |     |     |
| [reduce-initial-value](https://sonarsource.github.io/rspec/#/rspec/S6959/javascript)                    | "Array.reduce()" calls should include an initial value                                                                         | ✅  |     |     |     | 💭  |     |
| [redundant-type-aliases](https://sonarsource.github.io/rspec/#/rspec/S6564/javascript)                  | Redundant type aliases should not be used                                                                                      | ✅  |     |     |     |     |     |
| [regex-complexity](https://sonarsource.github.io/rspec/#/rspec/S5843/javascript)                        | Regular expressions should not be too complicated                                                                              | ✅  |     |     |     | 💭  |     |
| [regular-expr](https://sonarsource.github.io/rspec/#/rspec/S4784/javascript)                            | Using regular expressions is security-sensitive                                                                                |     | ✅  |     |     |     | ❌  |
| [rules-of-hooks](https://sonarsource.github.io/rspec/#/rspec/S6440/javascript)                          | React Hooks should be properly called                                                                                          | ✅  |     |     |     |     |     |
| [semi](https://sonarsource.github.io/rspec/#/rspec/S1438/javascript)                                    | Statements should end with semicolons                                                                                          |     | ✅  | 🔧  |     |     | ❌  |
| [session-regeneration](https://sonarsource.github.io/rspec/#/rspec/S5876/javascript)                    | A new session should be created during user authentication                                                                     | ✅  |     |     |     |     |     |
| [shorthand-property-grouping](https://sonarsource.github.io/rspec/#/rspec/S3499/javascript)             | Shorthand object properties should be grouped at the beginning or end of an object declaration                                 |     | ✅  |     |     |     |     |
| [single-char-in-character-classes](https://sonarsource.github.io/rspec/#/rspec/S6397/javascript)        | Character classes in regular expressions should not contain only one character                                                 | ✅  |     |     |     | 💭  |     |
| [single-character-alternation](https://sonarsource.github.io/rspec/#/rspec/S6035/javascript)            | Single-character alternations in regular expressions should be replaced with character classes                                 | ✅  |     |     |     | 💭  |     |
| [slow-regex](https://sonarsource.github.io/rspec/#/rspec/S5852/javascript)                              | Using slow regular expressions is security-sensitive                                                                           | ✅  |     |     |     | 💭  |     |
| [sockets](https://sonarsource.github.io/rspec/#/rspec/S4818/javascript)                                 | Using Sockets is security-sensitive                                                                                            |     | ✅  |     |     |     | ❌  |
| [sonar-block-scoped-var](https://sonarsource.github.io/rspec/#/rspec/S2392/javascript)                  | Variables should be used in the blocks where they are declared                                                                 | ✅  |     |     |     |     |     |
| [sonar-jsx-no-leaked-render](https://sonarsource.github.io/rspec/#/rspec/S6439/javascript)              | React components should not render non-boolean condition values                                                                | ✅  |     | 🔧  | 💡  | 💭  |     |
| [sonar-max-lines](https://sonarsource.github.io/rspec/#/rspec/S104/javascript)                          | Files should not have too many lines of code                                                                                   |     | ✅  |     |     |     |     |
| [sonar-max-lines-per-function](https://sonarsource.github.io/rspec/#/rspec/S138/javascript)             | Functions should not have too many lines of code                                                                               |     | ✅  |     |     |     |     |
| [sonar-max-params](https://sonarsource.github.io/rspec/#/rspec/S107/javascript)                         | Functions should not have too many parameters                                                                                  | ✅  |     |     |     |     |     |
| [sonar-no-control-regex](https://sonarsource.github.io/rspec/#/rspec/S6324/javascript)                  | Regular expressions should not contain control characters                                                                      | ✅  |     |     |     | 💭  |     |
| [sonar-no-dupe-keys](https://sonarsource.github.io/rspec/#/rspec/S1534/javascript)                      | Member names should not be duplicated within a class or object literal                                                         | ✅  |     | 🔧  | 💡  |     |     |
| [sonar-no-empty-character-class](https://sonarsource.github.io/rspec/#/rspec/S2639/javascript)          | Empty character classes should not be used                                                                                     | ✅  |     |     |     | 💭  |     |
| [sonar-no-fallthrough](https://sonarsource.github.io/rspec/#/rspec/S128/javascript)                     | Switch cases should end with an unconditional "break" statement                                                                | ✅  |     |     |     |     |     |
| [sonar-no-invalid-regexp](https://sonarsource.github.io/rspec/#/rspec/S5856/javascript)                 | Regular expressions should be syntactically valid                                                                              | ✅  |     |     |     | 💭  |     |
| [sonar-no-magic-numbers](https://sonarsource.github.io/rspec/#/rspec/S109/javascript)                   | Magic numbers should not be used                                                                                               |     | ✅  |     |     |     |     |
| [sonar-no-misleading-character-class](https://sonarsource.github.io/rspec/#/rspec/S5868/javascript)     | Unicode Grapheme Clusters should be avoided inside regex character classes                                                     | ✅  |     |     | 💡  | 💭  |     |
| [sonar-no-regex-spaces](https://sonarsource.github.io/rspec/#/rspec/S6326/javascript)                   | Regular expressions should not contain multiple spaces                                                                         | ✅  |     | 🔧  | 💡  | 💭  |     |
| [sonar-no-unused-class-component-methods](https://sonarsource.github.io/rspec/#/rspec/S6441/javascript) | Unused methods of React components should be removed                                                                           | ✅  |     |     |     |     |     |
| [sonar-no-unused-vars](https://sonarsource.github.io/rspec/#/rspec/S1481/javascript)                    | Unused local variables and functions should be removed                                                                         | ✅  |     |     |     |     |     |
| [sonar-prefer-optional-chain](https://sonarsource.github.io/rspec/#/rspec/S6582/javascript)             | Optional chaining should be preferred                                                                                          | ✅  |     | 🔧  | 💡  |     |     |
| [sonar-prefer-read-only-props](https://sonarsource.github.io/rspec/#/rspec/S6759/javascript)            | React props should be read-only                                                                                                | ✅  |     | 🔧  | 💡  | 💭  |     |
| [sonar-prefer-regexp-exec](https://sonarsource.github.io/rspec/#/rspec/S6594/javascript)                | "RegExp.exec()" should be preferred over "String.match()"                                                                      | ✅  |     | 🔧  | 💡  | 💭  |     |
| [sql-queries](https://sonarsource.github.io/rspec/#/rspec/S2077/javascript)                             | Formatting SQL queries is security-sensitive                                                                                   | ✅  |     |     |     |     |     |
| [stable-tests](https://sonarsource.github.io/rspec/#/rspec/S5973/javascript)                            | Tests should be stable                                                                                                         | ✅  |     |     |     |     |     |
| [standard-input](https://sonarsource.github.io/rspec/#/rspec/S4829/javascript)                          | Reading the Standard Input is security-sensitive                                                                               |     | ✅  |     |     |     | ❌  |
| [stateful-regex](https://sonarsource.github.io/rspec/#/rspec/S6351/javascript)                          | Regular expressions with the global flag should be used with caution                                                           | ✅  |     |     |     |     |     |
| [strict-transport-security](https://sonarsource.github.io/rspec/#/rspec/S5739/javascript)               | Disabling Strict-Transport-Security policy is security-sensitive                                                               | ✅  |     |     |     |     |     |
| [strings-comparison](https://sonarsource.github.io/rspec/#/rspec/S3003/javascript)                      | Comparison operators should not be used with strings                                                                           |     | ✅  |     |     | 💭  |     |
| [super-invocation](https://sonarsource.github.io/rspec/#/rspec/S3854/javascript)                        | "super()" should be invoked appropriately                                                                                      | ✅  |     |     |     |     |     |
| [switch-without-default](https://sonarsource.github.io/rspec/#/rspec/S131/javascript)                   | "switch" statements should have "default" clauses                                                                              |     | ✅  | 🔧  | 💡  | 💭  |     |
| [table-header](https://sonarsource.github.io/rspec/#/rspec/S5256/javascript)                            | Tables should have headers                                                                                                     | ✅  |     |     |     |     |     |
| [table-header-reference](https://sonarsource.github.io/rspec/#/rspec/S5260/javascript)                  | Table cells should reference their headers                                                                                     | ✅  |     |     |     |     |     |
| [test-check-exception](https://sonarsource.github.io/rspec/#/rspec/S5958/javascript)                    | Tests should check which exception is thrown                                                                                   | ✅  |     |     |     |     |     |
| [todo-tag](https://sonarsource.github.io/rspec/#/rspec/S1135/javascript)                                | Track uses of "TODO" tags                                                                                                      | ✅  |     |     |     |     |     |
| [too-many-break-or-continue-in-loop](https://sonarsource.github.io/rspec/#/rspec/S135/javascript)       | Loops should not contain more than a single "break" or "continue" statement                                                    |     | ✅  |     |     |     |     |
| [unicode-aware-regex](https://sonarsource.github.io/rspec/#/rspec/S5867/javascript)                     | Regular expressions using Unicode character classes or property escapes should enable the unicode flag                         |     | ✅  |     |     | 💭  |     |
| [unnecessary-character-escapes](https://sonarsource.github.io/rspec/#/rspec/S6535/javascript)           | Unnecessary character escapes should be removed                                                                                | ✅  |     | 🔧  | 💡  |     |     |
| [unused-import](https://sonarsource.github.io/rspec/#/rspec/S1128/javascript)                           | Unnecessary imports should be removed                                                                                          | ✅  |     | 🔧  | 💡  | 💭  |     |
| [unused-named-groups](https://sonarsource.github.io/rspec/#/rspec/S5860/javascript)                     | Names of regular expressions named groups should be used                                                                       | ✅  |     |     |     | 💭  |     |
| [unverified-certificate](https://sonarsource.github.io/rspec/#/rspec/S4830/javascript)                  | Server certificates should be verified during SSL/TLS connections                                                              | ✅  |     |     |     |     |     |
| [unverified-hostname](https://sonarsource.github.io/rspec/#/rspec/S5527/javascript)                     | Server hostnames should be verified during SSL/TLS connections                                                                 | ✅  |     |     |     |     |     |
| [updated-const-var](https://sonarsource.github.io/rspec/#/rspec/S3500/javascript)                       | "const" variables should not be reassigned                                                                                     | ✅  |     |     |     |     |     |
| [updated-loop-counter](https://sonarsource.github.io/rspec/#/rspec/S2310/javascript)                    | Loop counters should not be assigned within the loop body                                                                      | ✅  |     |     |     |     |     |
| [use-isnan](https://sonarsource.github.io/rspec/#/rspec/S2688/javascript)                               | "NaN" should not be used in comparisons                                                                                        | ✅  |     | 🔧  | 💡  |     |     |
| [use-type-alias](https://sonarsource.github.io/rspec/#/rspec/S4323/javascript)                          | Type aliases should be used                                                                                                    | ✅  |     |     |     |     |     |
| [useless-string-operation](https://sonarsource.github.io/rspec/#/rspec/S1154/javascript)                | Results of operations on strings should not be ignored                                                                         |     | ✅  |     |     | 💭  | ❌  |
| [values-not-convertible-to-numbers](https://sonarsource.github.io/rspec/#/rspec/S3758/javascript)       | Values not convertible to numbers should not be used in numeric comparisons                                                    |     | ✅  |     |     | 💭  |     |
| [variable-name](https://sonarsource.github.io/rspec/#/rspec/S117/javascript)                            | Variable, property and parameter names should comply with a naming convention                                                  |     | ✅  |     |     |     |     |
| [void-use](https://sonarsource.github.io/rspec/#/rspec/S3735/javascript)                                | "void" should not be used                                                                                                      | ✅  |     |     |     | 💭  |     |
| [weak-ssl](https://sonarsource.github.io/rspec/#/rspec/S4423/javascript)                                | Weak SSL/TLS protocols should not be used                                                                                      | ✅  |     |     |     |     |     |
| [web-sql-database](https://sonarsource.github.io/rspec/#/rspec/S2817/javascript)                        | Web SQL databases should not be used                                                                                           |     | ✅  |     |     | 💭  | ❌  |
| [x-powered-by](https://sonarsource.github.io/rspec/#/rspec/S5689/javascript)                            | Disclosing fingerprints from web application technologies is security-sensitive                                                | ✅  |     |     |     |     |     |
| [xml-parser-xxe](https://sonarsource.github.io/rspec/#/rspec/S2755/javascript)                          | XML parsers should not be vulnerable to XXE attacks                                                                            | ✅  |     |     |     |     |     |
| [xpath](https://sonarsource.github.io/rspec/#/rspec/S4817/javascript)                                   | Executing XPath expressions is security-sensitive                                                                              |     | ✅  |     |     |     | ❌  |

<!-- end auto-generated rules list -->

## Prerequisites

- Node.js (>=16.x).
- ESLint 8.x (version 9.X not yet supported)
- typescript-eslint/parser 7.X (version 8.X not yet supported)

## Usage

- If you don't have ESLint yet configured for your project, follow [these instructions](https://github.com/eslint/eslint#installation-and-usage).
- Install `eslint-plugin-sonarjs` using `npm` (or `yarn`) for your project or globally:

```sh
npm install eslint-plugin-sonarjs --save-dev # install for your project
npm install eslint-plugin-sonarjs -g         # or install globally
```

- Add `eslint-plugin-sonarjs` to the plugins of your ESLint config.

For ESLint 9: add `plugins` option to your `eslint.config.js` and include the recommended config to enable all rules:

```code
import sonarjs from "eslint-plugin-sonarjs";

[
  sonarjs.configs.recommended,
  {
    "plugins": {
      sonarjs,
    }
  }
]
```

For ESLint 8: add `plugins` option to your `.eslintrc` and `plugin:sonarjs/recommended-legacy` to the `extends` option to enable all recommended rules:

```json
{
  "plugins": ["sonarjs"],
  "extends": ["plugin:sonarjs/recommended-legacy"]
}
```

- or enable only some rules manually:

```json
{
  "rules": {
    "sonarjs/cognitive-complexity": "error",
    "sonarjs/no-identical-expressions": "error"
    // etc.
  }
}
```

- To allow each of the rules to fully perform, use `@typescript-eslint/parser` as a parser for ESLint and set the [parserOptions.project](https://github.com/typescript-eslint/typescript-eslint/tree/master/packages/parser#parseroptionsproject) option. This enables type checking which is beneficial or even essential for some rules.

## Available Configurations

This plugin provides only a `recommended` configuration. Almost all rules are activated in this profile with a few exceptions (check the `disabled` tag in the rules list). The `recommended` configuration activates rules with `error` severity.
We include a `recommended-legacy` configuration to be backward compatible with ESLint v8, with the same rule configuration..

## ESLint and Sonar

This plugin exposes to ESLint users all JS/TS rules from Sonar-\* products (aka [SonarJS](https://github.com/SonarSource/SonarJS)). We extracted the rules to be beneficial for the ESLint community.

If you are a [SonarQube](https://www.sonarqube.org) or [SonarCloud](https://sonarcloud.io) user, to lint your code locally, we suggest using [SonarLint](https://www.sonarlint.org) IDE extension (available for VSCode, JetBrains IDEs and Eclipse). You can connect SonarLint to your SonarQube/SonarCloud project to synchronize rules configuration, issue statuses, etc.
